from PIL import Image
import qrcode
class QrCodeServiceClass():
    def QrCode(self, codeContent, codeCoreImgPath=None, savePath=None, box_size = 8):
        qr = qrcode.QRCode(
            version=2,
            error_correction=qrcode.constants.ERROR_CORRECT_H,
            box_size=box_size,
            border=1
        )#各参数具体用法请参考底下链接
        qr.add_data(codeContent)#要显示的内容
        qr.make(fit=True)

        img = qr.make_image()
        img = img.convert("RGBA")
        if codeCoreImgPath:
            icon = Image.open(codeCoreImgPath)#打开一张图片作为二维码中心图标

            img_w, img_h = img.size
            factor = 4
            size_w = int(img_w / factor)
            size_h = int(img_h / factor)

            icon_w, icon_h = icon.size
            if icon_w > size_w:
                icon_w = size_w
            if icon_h > size_h:
                icon_h = size_h
            icon = icon.resize((icon_w, icon_h), Image.ANTIALIAS)

            w = int((img_w - icon_w) / 2)
            h = int((img_h - icon_h) / 2)

            img.paste(icon, (w, h))#将图标粘贴到二维码上
        img.save(savePath or "qrcode.png")#保存图片



import time
from datetime import datetime
from threading import Timer

class TimeServiceClass(object):
    def GetFileTime(self, mTime = None):
        return self.GetFormatTime('%Y_%m_%d_%H_%M_%S',mTime)
    
    def GetPrintTimeToAll(self, mTime = None):
        return self.GetFormatTime('%Y-%m-%d %H:%M:%S',mTime)
        
    def GetPrintTimeToDay(self, mTime = None):
        return self.GetFormatTime('%Y-%m-%d',mTime)

    def GetFormatTime(self, mFormat, mTime = None):
        if mTime == None:
            mTime = time.localtime()
        return time.strftime(mFormat,mTime)

    def GetLocalTime(self):
        return time.localtime()

    def GetTimeToInt(self):
        return int(time.time())

    def Sleep(self, sleep = 0):
        time.sleep(sleep)

    def TimeToStr(self, timeStamp, formatStr = "%Y--%m--%d %H:%M:%S"):
        timeArray = time.localtime(int(timeStamp))
        otherStyleTime = time.strftime(formatStr, timeArray)
        return otherStyleTime   # 2013--10--10 23:40:00

    """根据日期获取某天凌晨时间"""
    def GetDayZeroTime(self, date):
        if not date:
            return 0
        date_zero = datetime.now().replace(year=date.year, month=date.month,
                                                    day=date.day, hour=0, minute=0, second=0)
        date_zero_time = int(time.mktime(date_zero.timetuple())) * 1000
        return date_zero_time

    """根据日期获取当天凌晨时间  isSecond 是否是秒 """
    def GetCurrDayZeroTime(self, isSecond):
        # 今天的日期
        today_date = datetime.now().date()
        # 今天的零点
        ms = self.GetDayZeroTime(today_date) 
        if isSecond:
            return int(ms/1000)
        return ms



from FileService import FileServiceClass
class LocallizeServiceClass(object):
    def __init__(self, path = None):
        self.File = FileServiceClass()
        self.localData = {}
        self.ChangeData(path)

    def ChangeData(self, path=None):
        if path != None:
            if not self.File.Exists(path):
                print("Not Locallize File ->  {}".format(path))
            else:
                self.localData = self.File.SpliteContentKV(path)
                
    def Get(self, key):
        return self.localData.get(key or "NoLocallize",'{}{}'.format(str(key)," 没有写入到文本"))


# 好强大的 pyautogui， 能根据图识别图,然后循环操作
# 可以参考： https://blog.csdn.net/yzy_1996/article/details/85244714
# 可以考虑 算手机分辨率，然后实现手机点击
import cv2
import aircv as ac
import pyautogui
import os, time
import jieba
from PIL import Image
# from appium import webadb
from pytesseract import image_to_string
from ADBService import ADBServiceClass

class AIServiceClass():

    def __init__(self):
        self.adbService = ADBServiceClass()

    def img_contain_img_get_pos_list(self, img1, img2): 
        ''' 
        查找img2在img1的位置（坐标从左上角开始）
        return 匹配到的所有的列表(第一个列表相识度最高)
        '''
        imsrc = ac.imread(img1) # 原始图像
        imsch = ac.imread(img2) # 带查找的部分
        match_result=ac.find_all_template(imsrc, imsch,0.80)
        #提取出中心点的横纵坐标
        points=[]
        for i in match_result:
            # [result点坐标，rectangle四个角坐标=>左上坐下右上右下，confidence匹配度=>0.8以上基本就是了]
            points.append((i['result']))
        return points
    
    def img_contain_img_click_pos(self, img1, img2, isWin, isMultiTouch, click_num=1):
        ''' 
        点击img2在img1的位置
        isWin: 电脑是分辨率，手机需要换算分辨率和屏幕大小
        isMultiTouch: 是否是多点触控（所以满足的点，都点击）
        click_num: 点击次数
        return {state:是否点击成功,msg:消息提示}
        '''
        resultData = {'state': False, 'msg':''}
        pos_list = self.img_contain_img_get_pos_list(img1, img2)
        if not pos_list or len(pos_list)==0:
            resultData['msg'] = '没有匹配到对应图'
        else:
            for item in pos_list[:]:
                if isWin:
                    pyautogui.moveTo(item[0],item[1])
                    pyautogui.click(clicks=click_num, interval=0.2)
                else:
                    for clickIndex in range(0, click_num):
                        self.adbService.Click(item[0],item[1])
                        time.sleep(0.2)
                if not isMultiTouch:
                    break
            resultData['state'] = True
        return resultData

    def img_contain_text_get_pos_list(self, text, img, isRemoveOutBox=True, isShowLog=False):
        '''
        instruction:
        基于tesseract识别出文字(不同语言有不同的文字识别库，此处用的是中文库 chi_sim，主要用于识别中文 )，
        返回文字在屏幕上的坐标点列表。

        usage:
        text = '带查找文字'
        img = '图片地址.png'
        isRemoveOutBox = '结束是否移除数据文件'

        return 匹配到的所有的列表(第一个列表相识度最高)
        '''
        outSuffix = '.box'
        outName='out'
        outFullName='%s%s'%(outName, outSuffix)

        if os.path.exists(img):
            strValue = image_to_string(img, lang='chi_sim')
            if isShowLog:
                print(strValue)
            os.system('tesseract %s %s -l chi_sim makebox'%(img, outName))
            print('输出坐标文件 : %s'%outFullName)
        else:
            print('%s is not exists'%img)

        pos_list = []
        if os.path.exists(outFullName):
            with open(outFullName,'r', encoding='UTF-8') as f:
                for line in f:
                    if line.split()[0] in text:
                        pos_list.append(line.split())

        if os.path.exists(outFullName) and isRemoveOutBox:
            os.remove(outFullName)

        return pos_list

    def img_contain_text_click_pos(self, text, img, isWin, click_num=1, isHorizontal=True, isRemoveImg=True, isRemoveOutBox=True, isShowLog=False):
        '''
        点击文字在图片中的位置
        基于tesseract识别出文字(不同语言有不同的文字识别库，此处用的是中文库 chi_sim，主要用于识别中文 )，
        return {state:是否点击成功,msg:消息提示}

        usage:
        text = '带查找文字'
        img = '图片地址.png'
        isWin: 电脑是分辨率，手机需要换算分辨率和屏幕大小
        click_num: 点击次数
        isRemoveImg = '结束是否移除Img'
        isRemoveOutBox = '结束是否移除数据文件'
        '''

        resultData = {'state': False, 'msg':''}
        pos_list = self.img_contain_text_get_pos_list(text, img, isRemoveOutBox)

        if not pos_list or len(pos_list) == 0:
            resultData['msg'] = '没有匹配到对应字'
        else:
            if isShowLog:
                print(pos_list)
            xCenterPointList, yCenterPointList, xCenterPointSum, yCenterPointSum, x, y = [], [], 0, 0, 0, 0
            for item in pos_list[:]:
                tempXCenterPoint = (int(item[1]) + int(item[3]))/2
                xCenterPointList.append(tempXCenterPoint)
                xCenterPointSum = xCenterPointSum + tempXCenterPoint

                tempYCenterPoint = (int(item[2]) + int(item[4]))/2
                yCenterPointList.append(tempYCenterPoint)
                yCenterPointSum = yCenterPointSum + tempYCenterPoint

            x = int(xCenterPointSum/2)
            imgInstance = Image.open(img)
            height = imgInstance.size[1]    # 获取屏幕高度

            if isHorizontal:
                # 水平，一般认为三个字的中间点Y轴是一样的，所以取一个字的Y轴就可以了
                y = int((height - int(pos_list[0][2])) + (height - int(pos_list[0][4])))/2
            else:
                print('纵向文字的高计算有问题，记录记录，后面修')
                y = int(yCenterPointSum/2)

            if isShowLog:
                print('X--> %s   Y--> %s'%(x, y))
            if isWin:
                pyautogui.moveTo(x, y)
                pyautogui.click(clicks=click_num, interval=0.2)
            else:
                for clickIndex in range(0, click_num):
                    self.adbService.Click(x, y)
                    time.sleep(0.2)
            resultData['state'] = True
        if  os.path.exists(img) and isRemoveImg:
            os.remove(img)
        return resultData

    def img_contain_text(self, img, text, lang='chi_sim'):
        '''
        图片是否包含文字
        return bool
        '''
        resultStr = image_to_string(img, lang=lang)
        resultStr = resultStr.replace('\n', '').replace(' ', '')
        if type(text) == type([]):
            for item in text[:]:
                if item in resultStr:
                    return True
        else:
            if text in resultStr:
                return True
        return False

    def img_contain_text_jieba(self, img, text, lang='chi_sim', confidence=None, cut_all=False):
        '''
        图片是否包含某一句话（结巴）
        confidence 匹配度
        return 【传入匹配度，返回bool】【不传入匹配度，返回匹配度（0-1）】
        '''
        seg_list = jieba.cut(text)
        if not seg_list:
            return 0
        resultStr = image_to_string(img, lang=lang)
        resultStr = resultStr.replace('\n', '').replace(' ', '')
        used, sum = 0, 0
        for item in seg_list:
            if item[0] in resultStr:
                used += 1
            sum += 1
        confidenceT = used/sum
        if confidence:
            return confidenceT >= confidence
        else:
            return confidenceT

if __name__ == '__main__':
    # # GUI点击 clicks 点击次数
    # pyautogui.click(x=1596 ,y=1063,clicks=1, interval=2)
    # # GUI滚动鼠标
    # pyautogui.scroll(-300, x=100, y=100)
    # # 鼠标移动
    # pyautogui.moveTo(50,50)
    # # 截屏

    aiService = AIServiceClass()

    # adb = AdbTools()
    # screenshotPath = adb.screenshot('TestRes/screenshot.png')
    # print(screenshotPath)
    # print(aiService.img_contain_img_click_pos('TestRes/screenshot.png', 'TestRes/screenshotPathItem.png', False, False, 1))   #True
    # print(aiService.img_contain_img_click_pos('TestRes/1.png','TestRes/test.png', True, False, 1))   #True
    print(aiService.img_contain_img_click_pos('TestRes/Screenshot_DouYin.png','TestRes/Screenshot_DouYin_item.png', False, False, 1))   #False

    # print('-----------------------TestRes/Screenshot_DouYin.jpg-------------------------')
    # print(image_to_string('TestRes/Screenshot_DouYin.jpg', lang='chi_sim'))   #True
    # print('-----------------------TestRes/Screenshot_DouYin.png-------------------------')
    # print(image_to_string('TestRes/Screenshot_DouYin.png', lang='chi_sim'))   #True
    # print(aiService.img_contain_text('TestRes/Screenshot_DouYin.png', '儿童'))   #True
    print(aiService.img_contain_text_jieba('TestRes/Screenshot_DouYin.png', '为呵护未成年人健康成长,抖音推出'))     #True
    print(aiService.img_contain_text_jieba('TestRes/Screenshot_DouYin.png', '为呵护未成年人健康成长,抖音推出XXXX'))     #True

    # print("文字识别有问题，还很慢")
    # print(aiService.img_contain_text_click_pos('打开的编辑器', 'TestRes/AAA.png', True, 1, isRemoveImg=False, isShowLog=True))   #False


import time, os, pretty_errors, sys
from TimeService import TimeServiceClass
from FileService import FileServiceClass

class LogServiceClass(object):
    TimeService = TimeServiceClass()
    FileService = FileServiceClass()

    filePath = "{}{}-Common.log".format("./Log/",TimeService.GetPrintTimeToDay()) 
    tag = ""
    def __init__(self, filePath = None, tag = None):
        if filePath != None:
            self.filePath = filePath
        if tag != None:
            self.tag = tag
        self.FileService.AppendFile(self.filePath, "{}{} {} {}".format("\n", self.TimeService.GetPrintTimeToAll(), self.tag, "- Start \n\n"))
        
    # 打印日志
    def Log(self, content, type=0):
        front = "LogD"
        if type == 0:
            front = "LogD"
        elif type == 1:
            front = "LogW"
        elif type == 2:
            front = "LogE"
        outPrint = "{} {} {}: {}".format(self.TimeService.GetPrintTimeToAll(), self.tag, front, content)
        self.SavePrintToFile(outPrint)
        print(outPrint)

    def LogD(self, *content):
        self.Log(content, 0)

    def LogW(self, *content):
        self.Log(content, 1)

    def LogE(self, *content):
        self.Log(content, 2)

    def SavePrintToFile(self, content):
        if self.filePath:
            self.FileService.AppendFile(self.filePath, content, True)

    def LogProgressBar(self, portion, total, desc = ''):
        '''
        total 总数据大小，portion 已经传送的数据大小
        :param portion: 已经接收的数据量
        :param total: 总数据量
        :return: 接收数据完成，返回True
        '''
        part = total / 50  # 1%数据的大小
        count = int(portion / part)
        sys.stdout.write('\r')
        sys.stdout.write(desc+('[%-50s]%.2f%%' % (('>' * count), portion / total * 100)))
        sys.stdout.flush()

        if portion >= total:
            sys.stdout.write('\n')
            return True

if __name__ == '__main__':
    LogService = LogServiceClass(tag="ADBServiceClass")
    LogService.Log(15)
    LogService.LogD(16,66)
    LogService.LogW(17)
    LogService.LogE(18)


import os, hashlib
from natsort import natsorted
from shutil import copyfile

class FileServiceClass(object):
    def __init__(self):
        pass
    
    def AppendFile(self, filePath, content, isEnter = False):
        currDirName = os.path.dirname(filePath)
        if not os.path.exists(currDirName):
            os.makedirs(currDirName)
        fd = os.open(filePath,os.O_RDWR|os.O_APPEND|os.O_CREAT)
        os.write(fd,bytes(content + ("\n" if isEnter else ""), encoding='utf-8'))
        os.close(fd)

    def WriteFile(self, filePath, content, isEnter = False):
        currDirName = os.path.dirname(filePath)
        if not os.path.exists(currDirName):
            os.makedirs(currDirName)
        fd = os.open(filePath,os.O_RDWR|os.O_CREAT)
        os.write(fd,bytes(content + ("\n" if isEnter else ""), encoding='utf-8'))
        os.close(fd)

    def ReadFile(self, filePath, encoding = "utf-8"):
        configFs = open(filePath,"r+",encoding = encoding)
        configContent = configFs.read()
        return configContent

    def Exists(self, filePath):
        return os.path.exists(filePath)
    
    def GetFiles(self, dirPath, isCurrDir, excludeList = []):
        filesTemp = []
        for root, dirs, files in os.walk(dirPath):
            for file in files:
                isAppend = False
                if isCurrDir:
                    if root == dirPath:
                        isAppend = True
                else:
                    isAppend = True
                if isAppend and not file in excludeList:
                    filesTemp.append(os.path.join(root, file))
            if isCurrDir and root == dirPath:
                break
        return filesTemp
    
    def GetDirs(self,dirPath, isCurrDir, excludeList = []):
        dirsTemp = []
        for root, dirs, files in os.walk(dirPath):
            for dir in dirs:
                isAppend = False
                if isCurrDir:
                    if root == dirPath:
                        isAppend = True
                else:
                    isAppend = True
                if isAppend and not dir in excludeList:
                    dirsTemp.append(os.path.join(root, dir))
            if isCurrDir and root == dirPath:
                break
        return dirsTemp
    
    def CopyFile(self, sourceFile, targetFile):
        try:
            copyfile(sourceFile, targetFile)
        except IOError as e:
            print('CopyFile Error %s'%(str(e)))

    def GetFileMd5(self, filePath):
        if not os.path.isfile(filePath):
            return None
        myhash = hashlib.md5()
        f = open(filePath,'rb')
        while True:
            b = f.read(8096)
            if not b :
                break
            myhash.update(b)
        f.close()
        return myhash.hexdigest()

    def ShortFiles(self, fileList):
        return natsorted(fileList)

    def SpliteContentKV(self, path, spliteStr = "="):
        # spliteStrs 按照前后顺序切，也就是切出来是 表中表
        content = self.ReadFile(path)
        linesContent = content.splitlines()
        contents = {}
        for line in linesContent:
            splites = line.split(spliteStr, 1)
            contents.setdefault(splites[0].strip(),splites[1].strip())
        return contents
               
if __name__ == "__main__":
    print(FileServiceClass().SpliteContentKV("Locallize.txt","="))


print(" Init zqpy core By ZhouQing")

import os, sys, json
sys.path.append(os.path.dirname(__file__))

from .LogService import LogServiceClass
from .FileService import FileServiceClass
from .HttpService import HttpServiceClass
from .RegexService import RegexServiceClass
from .ThreadService import ThreadServiceClass
from .TimeService import TimeServiceClass
from .VideoDownloadService import VideoDownloadServiceClass
from .LocallizeService import LocallizeServiceClass
from .WaitExecutService import WaitExecutServiceClass
from .QrCodeService import QrCodeServiceClass
from .ToolsService import ToolsServiceClass
from .MailService import MailServiceClass
from .ADBService import *
from .SqliteService import SqliteServiceClass
from .AIService import AIServiceClass

def OpenAutoInstall():
    from .AutoInstall import AutoInstallClass# 自动导入缺失的库

def vGetLogTag():
    return "BasePyClass"

def vLocallizePath():
    return None

Log = LogServiceClass(tag=vGetLogTag())
LogD = Log.LogD
LogW = Log.LogW
LogE = Log.LogE
FileService = FileServiceClass()
HttpService = HttpServiceClass()
RegexService = RegexServiceClass()
ThreadService = ThreadServiceClass()
TimeService = TimeServiceClass()
VideoDownloadService = VideoDownloadServiceClass()
LocalizeService = LocallizeServiceClass(path=(vLocallizePath() or None))
WaitExecutService = WaitExecutServiceClass()
QrCodeService = QrCodeServiceClass()
ToolsService = ToolsServiceClass()
MailService = MailServiceClass()
ADBService = ADBServiceClass()
SqliteService = SqliteServiceClass()

AIService = AIServiceClass()

########################################通用方法##########################################
def GetLocalize(key):
    return LocalizeService.Get(key)

############################################子类可重写##########################################


# 以下代码在 python 3.6.1 版本验证通过import sysimport osfrom importlib import import_module
import sys, os
from importlib import import_module
class AutoInstallClass():
    _loaded = set()
    @classmethod
    def find_spec(cls, name, path, target=None):
        if path is None and name not in cls._loaded:
            cls._loaded.add(name)
            print("Installing", name)
            try:
                result = os.system('pip install {}'.format(name))
                if result == 0:
                    return import_module(name)
            except Exception as e:
                print("Failed", e)
                return None
print('引入自动导入库成功')
sys.meta_path.append(AutoInstallClass)



#!/usr/bin/evn python
# -*- coding:utf-8 -*-
 
# FileName adbtools.py
# Author: HeyNiu
# Created Time: 2016/9/19
"""
adb 工具类
"""
 
import os
import platform
import re
import time
class AdbTools(object):

    def __init__(self, device_id=''):
        self.__system = platform.system()
        self.__find = ''
        self.__command = r'F:\Tools\ADB\ADB\adb.exe'
        self.__device_id = device_id
        self.__get_find()
        self.__check_adb()
        self.__connection_devices()
 
    def __get_find(self):
        """
        判断系统类型，windows使用findstr，linux使用grep
        :return:
        """
        if self.__system == "Windows":
            self.__find = "findstr"
        else:
            self.__find = "grep"
 
    def __check_adb(self):
        """
        检查adb
        判断是否设置环境变量ANDROID_ADB
        :return:
        """
        '''
        if "ANDROID_ADB" in os.environ:
            if self.__system == "Windows":
                # path = os.path.join(os.environ["ANDROID_ADB"], "platform-tools", "adb.exe")
                path = os.path.join(os.environ["ANDROID_ADB"], "adb.exe")
                if os.path.exists(path):
                    self.__command = path
                else:
                    raise EnvironmentError(
                        "Adb not found in $ANDROID_ADB path: %s." % os.environ["ANDROID_ADB"])
            else:
                path = os.path.join(os.environ["ANDROID_ADB"], "adb")
                if os.path.exists(path):
                    self.__command = path
                else:
                    raise EnvironmentError(
                        "Adb not found in $ANDROID_ADB path: %s." % os.environ["ANDROID_ADB"])
        else:
            raise EnvironmentError(
                "Adb not found in $ANDROID_ADB path: %s." % os.environ["ANDROID_ADB"])
        '''
        return True
 
    def __connection_devices(self):
        """
        连接指定设备，单个设备可不传device_id
        :return:
        """
        if self.__device_id == "":
            return
        self.__device_id = "-s %s" % self.__device_id
 
    def adb(self, args):
        """
        执行adb命令
        :param args:参数
        :return:
        """
        cmd = "%s %s %s" % (self.__command, self.__device_id, str(args))
        # print(cmd)
        return os.popen(cmd)
 
    def shell(self, args):
        """
        执行adb shell命令
        :param args:参数
        :return:
        """
        cmd = "%s %s shell %s" % (self.__command, self.__device_id, str(args))
        # print(cmd)
        return os.popen(cmd)
 
    def mkdir(self, path):
        """
        创建目录
        :param path: 路径
        :return:
        """
        return self.shell('mkdir %s' % path)
 
    def get_devices(self):
        """
        获取设备列表
        :return:
        """
        l = self.adb('devices').readlines()
        return (i.split()[0] for i in l if 'devices' not in i and len(i) > 5)
 
    def get_current_application(self):
        """
        获取当前运行的应用信息
        :return:
        """
        return self.shell('dumpsys window w | %s \/ | %s name=' % (self.__find, self.__find)).read()
 
    def get_current_package(self):
        """
        获取当前运行app包名
        :return:
        """
        reg = re.compile(r'name=(.+?)/')
        return re.findall(reg, self.get_current_application())[0]
 
    def get_current_activity(self):
        """
        获取当前运行activity
        :return: package/activity
        """
        reg = re.compile(r'name=(.+?)\)')
        return re.findall(reg, self.get_current_application())[0]
 
    def __get_process(self, package_name):
        """
        获取进程信息
        :param package_name:
        :return:
        """
        if self.__system == "Windows":
            pid_command = self.shell("ps | %s %s$" % (self.__find, package_name)).read()
        else:
            pid_command = self.shell("ps | %s -w %s" % (self.__find, package_name)).read()
        return pid_command
 
    def process_exists(self, package_name):
        """
        返回进程是否存在
        :param package_name:
        :return:
        """
        process = self.__get_process(package_name)
        return package_name in process
 
    def get_pid(self, package_name):
        """
        获取pid
        :return:
        """
        pid_command = self.__get_process(package_name)
        if pid_command == '':
            print("The process doesn't exist.")
            return pid_command
 
        req = re.compile(r"\d+")
        result = str(pid_command).split()
        result.remove(result[0])
        return req.findall(" ".join(result))[0]
 
    def get_uid(self, pid):
        """
        获取uid
        :param pid:
        :return:
        """
        result = self.shell("cat /proc/%s/status" % pid).readlines()
        for i in result:
            if 'uid' in i.lower():
                return i.split()[1]
 
    def get_flow_data_tcp(self, uid):
        """
        获取应用tcp流量
        :return:(接收, 发送)
        """
        tcp_rcv = self.shell("cat proc/uid_stat/%s/tcp_rcv" % uid).read().split()[0]
        tcp_snd = self.shell("cat proc/uid_stat/%s/tcp_snd" % uid).read().split()[0]
        return tcp_rcv, tcp_snd
 
    def get_flow_data_all(self, uid):
        """
        获取应用流量全部数据
        包含该应用多个进程的所有数据 tcp udp等
        (rx_bytes, tx_bytes) >> (接收, 发送)
        :param uid:
        :return:list(dict)
        """
        all_data = []
        d = {}
        data = self.shell("cat /proc/net/xt_qtaguid/stats | %s %s" % (self.__find, uid)).readlines()
        for i in data:
            if not i.startswith('\n'):
                item = i.strip().split()
                d['idx'] = item[0]
                d['iface'] = item[1]
                d['acct_tag_hex'] = item[2]
                d['uid_tag_int'] = item[3]
                d['cnt_set'] = item[4]
                d['rx_bytes'] = item[5]
                d['rx_packets'] = item[6]
                d['tx_bytes'] = item[7]
                d['tx_packets'] = item[8]
                d['rx_tcp_bytes'] = item[9]
                d['rx_tcp_packets'] = item[10]
                d['rx_udp_bytes'] = item[11]
                d['rx_udp_packets'] = item[12]
                d['rx_other_bytes'] = item[13]
                d['rx_other_packets'] = item[14]
                d['tx_tcp_bytes'] = item[15]
                d['tx_tcp_packets'] = item[16]
                d['tx_udp_bytes'] = item[17]
                d['tx_udp_packets'] = item[18]
                d['tx_other_bytes'] = item[19]
                d['tx_other_packets'] = item[20]
 
                all_data.append(d)
                d = {}
        return all_data
 
    @staticmethod
    def dump_apk(path):
        """
        dump apk文件
        :param path: apk路径
        :return:
        """
        # 检查build-tools是否添加到环境变量中
        # 需要用到里面的aapt命令
        l = os.environ['PATH'].split(';')
        build_tools = False
        for i in l:
            if 'build-tools' in i:
                build_tools = True
        if not build_tools:
            raise EnvironmentError("ANDROID_ADB BUILD-TOOLS COMMAND NOT FOUND.\nPlease set the environment variable.")
        return os.popen('aapt dump badging %s' % (path,))
 
    @staticmethod
    def dump_xml(path, filename):
        """
        dump apk xml文件
        :return:
        """
        return os.popen('aapt dump xmlstrings %s %s' % (path, filename))
 
    def uiautomator_dump(self):
        """
        获取屏幕uiautomator xml文件
        :return:
        """
        return self.shell('uiautomator dump').read().split()[-1]
 
    def pull(self, source, target):
        """
        从手机端拉取文件到电脑端
        :return:
        """
        self.adb('pull %s %s' % (source, target))
 
    def push(self, source, target):
        """
        从电脑端推送文件到手机端
        :param source:
        :param target:
        :return:
        """
        self.adb('push %s %s' % (source, target))
 
    def remove(self, path):
        """
        从手机端删除文件
        :return:
        """
        self.shell('rm %s' % (path,))
 
    def clear_app_data(self, package):
        """
        清理应用数据
        :return:
        """
        self.shell('pm clear %s' % (package,))
 
    def install(self, path):
        """
        安装apk文件
        :return:
        """
        # adb install 安装错误常见列表
        errors = {'INSTALL_FAILED_ALREADY_EXISTS': '程序已经存在',
                  'INSTALL_DEVICES_NOT_FOUND': '找不到设备',
                  'INSTALL_FAILED_DEVICE_OFFLINE': '设备离线',
                  'INSTALL_FAILED_INVALID_APK': '无效的APK',
                  'INSTALL_FAILED_INVALID_URI': '无效的链接',
                  'INSTALL_FAILED_INSUFFICIENT_STORAGE': '没有足够的存储空间',
                  'INSTALL_FAILED_DUPLICATE_PACKAGE': '已存在同名程序',
                  'INSTALL_FAILED_NO_SHARED_USER': '要求的共享用户不存在',
                  'INSTALL_FAILED_UPDATE_INCOMPATIBLE': '版本不能共存',
                  'INSTALL_FAILED_SHARED_USER_INCOMPATIBLE': '需求的共享用户签名错误',
                  'INSTALL_FAILED_MISSING_SHARED_LIBRARY': '需求的共享库已丢失',
                  'INSTALL_FAILED_REPLACE_COULDNT_DELETE': '需求的共享库无效',
                  'INSTALL_FAILED_DEXOPT': 'dex优化验证失败',
                  'INSTALL_FAILED_DEVICE_NOSPACE': '手机存储空间不足导致apk拷贝失败',
                  'INSTALL_FAILED_DEVICE_COPY_FAILED': '文件拷贝失败',
                  'INSTALL_FAILED_OLDER_SDK': '系统版本过旧',
                  'INSTALL_FAILED_CONFLICTING_PROVIDER': '存在同名的内容提供者',
                  'INSTALL_FAILED_NEWER_SDK': '系统版本过新',
                  'INSTALL_FAILED_TEST_ONLY': '调用者不被允许测试的测试程序',
                  'INSTALL_FAILED_CPU_ABI_INCOMPATIBLE': '包含的本机代码不兼容',
                  'CPU_ABIINSTALL_FAILED_MISSING_FEATURE': '使用了一个无效的特性',
                  'INSTALL_FAILED_CONTAINER_ERROR': 'SD卡访问失败',
                  'INSTALL_FAILED_INVALID_INSTALL_LOCATION': '无效的安装路径',
                  'INSTALL_FAILED_MEDIA_UNAVAILABLE': 'SD卡不存在',
                  'INSTALL_FAILED_INTERNAL_ERROR': '系统问题导致安装失败',
                  'INSTALL_PARSE_FAILED_NO_CERTIFICATES': '文件未通过认证 >> 设置开启未知来源',
                  'INSTALL_PARSE_FAILED_INCONSISTENT_CERTIFICATES': '文件认证不一致 >> 先卸载原来的再安装',
                  'INSTALL_FAILED_INVALID_ZIP_FILE': '非法的zip文件 >> 先卸载原来的再安装',
                  'INSTALL_CANCELED_BY_USER': '需要用户确认才可进行安装',
                  'INSTALL_FAILED_VERIFICATION_FAILURE': '验证失败 >> 尝试重启手机',
                  'DEFAULT': '未知错误'
                  }
        print('Installing...')
        l = self.adb('install -r %s' % (path,)).read()
        if 'Success' in l:
            print('Install Success')
        if 'Failure' in l:
            reg = re.compile('\\[(.+?)\\]')
            key = re.findall(reg, l)[0]
            try:
                print('Install Failure >> %s' % errors[key])
            except KeyError:
                print('Install Failure >> %s' % key)
        return l
 
    def uninstall(self, package):
        """
        卸载apk
        :param package: 包名
        :return:
        """
        print('Uninstalling...')
        l = self.adb('uninstall %s' % (package,)).read()
        print(l)
 
    def screenshot(self, target_path=''):
        """
        手机截图
        :param target_path: 目标路径
        :return:
        """
        format_time = int(time.time())
        self.shell('screencap -p /sdcard/%s.png' % (format_time,))
        time.sleep(2)
        outPath = target_path
        if target_path == '':
            outPath = os.path.dirname(__file__)#os.path.expanduser('~')
        self.pull('/sdcard/%s.png' % (format_time,), outPath)
        self.remove('/sdcard/%s.png' % (format_time,))
        time.sleep(1)
        return outPath if ('.jpg' in outPath or '.png' in outPath) else ('%s%s.png' % (outPath, format_time,))
 
    def get_cache_logcat(self):
        """
        导出缓存日志
        :return:
        """
        return self.adb('logcat -v time -d')
 
    def get_crash_logcat(self):
        """
        导出崩溃日志
        :return:
        """
        return self.adb('logcat -v time -d | %s AndroidRuntime' % (self.__find,))
 
    def clear_cache_logcat(self):
        """
        清理缓存区日志
        :return:
        """
        self.adb('logcat -c')
 
    def get_device_time(self):
        """
        获取设备时间
        :return:
        """
        return self.shell('date').read().strip()
 
    def ls(self, command):
        """
        shell ls命令
        :return:
        """
        return self.shell('ls %s' % (command,)).readlines()
 
    def file_exists(self, target):
        """
        判断文件在目标路径是否存在
        :return:
        """
        l = self.ls(target)
        for i in l:
            if i.strip() == target:
                return True
        return False
 
    def is_install(self, target_app):
        """
        判断目标app在设备上是否已安装
        :param target_app: 目标app包名
        :return: bool
        """
        return target_app in self.shell('pm list packages %s' % (target_app,)).read()
 
    def get_device_model(self):
        """
        获取设备型号
        :return:
        """
        return self.shell('getprop ro.product.model').read().strip()
 
    def get_device_id(self):
        """
        获取设备id
        :return:
        """
        return self.adb('get-serialno').read().strip()
 
    def get_device_android_version(self):
        """
        获取设备Android版本
        :return:
        """
        return self.shell('getprop ro.build.version.release').read().strip()
 
    def get_device_sdk_version(self):
        """
        获取设备SDK版本
        :return:
        """
        return self.shell('getprop ro.build.version.sdk').read().strip()
 
    def get_device_mac_address(self):
        """
        获取设备MAC地址
        :return:
        """
        return self.shell('cat /sys/class/net/wlan0/address').read().strip()
 
    def get_device_ip_address(self):
        """
        获取设备IP地址
        pass: 适用WIFI 蜂窝数据
        :return:
        """
        if not self.get_wifi_state() and not self.get_data_state():
            return
        l = self.shell('ip addr | %s global' % self.__find).read()
        reg = re.compile('\d+\.\d+\.\d+\.\d+')
        return re.findall(reg, l)[0]
 
    def get_device_imei(self):
        """
        获取设备IMEI
        :return:
        """
        sdk = self.get_device_sdk_version()
        # Android 5.0以下方法
        if int(sdk) < 21:
            l = self.shell('dumpsys iphonesubinfo').read()
            reg = re.compile('[0-9]{15}')
            return re.findall(reg, l)[0]
        elif self.root():
            l = self.shell('service call iphonesubinfo 1').read()
            print(l)
            print(re.findall(re.compile("'.+?'"), l))
            imei = ''
            for i in re.findall(re.compile("'.+?'"), l):
                imei += i.replace('.', '').replace("'", '').replace(' ', '')
            return imei
        else:
            print('The device not root.')
            return ''
 
    def check_sim_card(self):
        """
        检查设备SIM卡
        :return:
        """
        return len(self.shell('getprop | %s gsm.operator.alpha]' % self.__find).read().strip().split()[-1]) > 2
 
    def get_device_operators(self):
        """
        获取运营商
        :return:
        """
        return self.shell('getprop | %s gsm.operator.alpha]' % self.__find).read().strip().split()[-1]
 
    def get_device_state(self):
        """
        获取设备状态
        :return:
        """
        return self.adb('get-state').read().strip()
 
    def get_display_state(self):
        """
        获取屏幕状态
        :return: 亮屏/灭屏
        """
        l = self.shell('dumpsys power').readlines()
        for i in l:
            if 'mScreenOn=' in i:
                return i.split()[-1] == 'mScreenOn=true'
            if 'Display Power' in i:
                return 'ON' in i.split('=')[-1].upper()
 
    def get_screen_normal_size(self):
        """
        获取设备屏幕分辨率 >> 标配
        :return:
        """
        return self.shell('wm size').read().strip().split()[-1].split('x')
 
    def get_screen_reality_size(self):
        """
        获取设备屏幕分辨率 >> 实际分辨率
        :return:
        """
        x = 0
        y = 0
        l = self.shell(r'getevent -p | %s -e "0"' % self.__find).readlines()
        for n in l:
            if len(n.split()) > 0:
                if n.split()[0] == '0035':
                    x = int(n.split()[7].split(',')[0])
                elif n.split()[0] == '0036':
                    y = int(n.split()[7].split(',')[0])
        return x, y
 
    def get_device_interior_sdcard(self):
        """
        获取内部SD卡空间
        :return: (path,total,used,free,block)
        """
        return self.shell('df | %s \/mnt\/shell\/emulated' % self.__find).read().strip().split()
 
    def get_device_external_sdcard(self):
        """
        获取外部SD卡空间
        :return: (path,total,used,free,block)
        """
        return self.shell('df | %s \/storage' % self.__find).read().strip().split()
 
    def __fill_rom(self, path, stream, count):
        """
        填充数据
        :param path: 填充地址
        :param stream: 填充流大小
        :param count: 填充次数
        :return:
        """
        self.shell('dd if=/dev/zero of=%s bs=%s count=%s' % (path, stream, count)).read().strip()
 
    def fill_interior_sdcard(self, filename, size):
        """
        填充内置SD卡
        :param filename: 文件名
        :param size: 填充大小，单位byte
        :return:
        """
        if size > 10485760:  # 10m
            self.__fill_rom('sdcard/%s' % filename, 10485760, size / 10485760)
        else:
            self.__fill_rom('sdcard/%s' % filename, size, 1)
 
    def fill_external_sdcard(self, filename, size):
        """
        填充外置SD卡
        :param filename: 文件名
        :param size: 填充大小，单位byte
        :return:
        """
        path = self.get_device_external_sdcard()[0]
        if size > 10485760:  # 10m
            self.__fill_rom('%s/%s' % (path, filename), 10485760, size / 10485760)
        else:
            self.__fill_rom('%s/%s' % (path, filename), size, 1)
 
    def kill_process(self, pid):
        """
        杀死进程
        pass: 一般需要权限不推荐使用
        :return:
        """
        return self.shell('kill %s' % pid).read().strip()
 
    def quit_app(self, package):
        """
        退出应用
        :return:
        """
        return self.shell('am force-stop %s' % package).read().strip()
 
    def reboot(self):
        """
        重启设备
        :return:
        """
        self.adb('reboot')
 
    def recovery(self):
        """
        重启设备并进入recovery模式
        :return:
        """
        self.adb('reboot recovery')
 
    def fastboot(self):
        """
        重启设备并进入fastboot模式
        :return:
        """
        self.adb('reboot bootloader')
 
    def root(self):
        """
        获取root状态
        :return:
        """
        return 'not found' not in self.shell('su -c ls -l /data/').read().strip()
 
    def wifi(self, power):
        """
        开启/关闭wifi
        pass: 需要root权限
        :return:
        """
        if not self.root():
            print('The device not root.')
            return
        if power:
            self.shell('su -c svc wifi enable').read().strip()
        else:
            self.shell('su -c svc wifi disable').read().strip()
 
    def data(self, power):
        """
        开启/关闭蜂窝数据
        pass: 需要root权限
        :return:
        """
        if not self.root():
            print('The device not root.')
            return
        if power:
            self.shell('su -c svc data enable').read().strip()
        else:
            self.shell('su -c svc data disable').read().strip()
 
    def get_wifi_state(self):
        """
        获取WiFi连接状态
        :return:
        """
        return 'enabled' in self.shell('dumpsys wifi | %s ^Wi-Fi' % self.__find).read().strip()
 
    def get_data_state(self):
        """
        获取移动网络连接状态
        :return:
        """
        return '2' in self.shell('dumpsys telephony.registry | %s mDataConnectionState' % self.__find).read().strip()
 
    def get_network_state(self):
        """
        设备是否连上互联网
        :return:
        """
        return 'unknown host' not in self.shell('ping -w 1 www.baidu.com').read().strip()
 
    def get_wifi_password_list(self):
        """
        获取WIFI密码列表
        :return:
        """
        if not self.root():
            print('The device not root.')
            return []
        l = re.findall(re.compile('ssid=".+?"\s{3}psk=".+?"'), self.shell('su -c cat /data/misc/wifi/*.conf').read())
        return [re.findall(re.compile('".+?"'), i) for i in l]
 
    def call(self, number):
        """
        拨打电话
        :param number:
        :return:
        """
        self.shell('am start -a android.intent.action.CALL -d tel:%s' % number)
 
    def open_url(self, url):
        """
        打开网页
        :return:
        """
        self.shell('am start -a android.intent.action.VIEW -d %s' % url)
 
    def start_application(self, component):
        """
        启动一个应用
        e.g: com.android.settings/com.android.settings.Settings
        """
        self.shell("am start -n %s" % component)
 
    def send_keyevent(self, keycode):
        """
        发送一个按键事件
        https://developer.android.com/reference/android/view/KeyEvent.html
        :return:
        """
        self.shell('input keyevent %s' % keycode)
 
    def rotation_screen(self, param):
        """
        旋转屏幕
        :param param: 0 >> 纵向，禁止自动旋转; 1 >> 自动旋转
        :return:
        """
        self.shell('/system/bin/content insert --uri content://settings/system --bind '
                   'name:s:accelerometer_rotation --bind value:i:%s' % param)
 
    def instrument(self, command):
        """
        启动instrument app
        :param command: 命令
        :return:
        """
        return self.shell('am instrument %s' % command).read()
 
    def export_apk(self, package, target_path='', timeout=5000):
        """
        从设备导出应用
        :param timeout: 超时时间
        :param target_path: 导出后apk存储路径
        :param package: 包名
        :return:
        """
        num = 0
        if target_path == '':
            self.adb('pull /data/app/%s-1/base.apk %s' % (package, os.path.expanduser('~')))
            while 1:
                num += 1
                if num <= timeout:
                    if os.path.exists(os.path.join(os.path.expanduser('~'), 'base.apk')):
                        os.rename(os.path.join(os.path.expanduser('~'), 'base.apk'),
                                  os.path.join(os.path.expanduser('~'), '%s.apk' % package))
 
        else:
            self.adb('pull /data/app/%s-1/base.apk %s' % (package, target_path))
            while 1:
                num += 1
                if num <= timeout:
                    if os.path.exists(os.path.join(os.path.expanduser('~'), 'base.apk')):
                        os.rename(os.path.join(os.path.expanduser('~'), 'base.apk'),
                                  os.path.join(os.path.expanduser('~'), '%s.apk' % package))
 
 
class KeyCode:
    KEYCODE_CALL = 5  # 拨号键
    KEYCODE_ENDCALL = 6  # 挂机键
    KEYCODE_HOME = 3  # Home键
    KEYCODE_MENU = 82  # 菜单键
    KEYCODE_BACK = 4  # 返回键
    KEYCODE_SEARCH = 84  # 搜索键
    KEYCODE_CAMERA = 27  # 拍照键
    KEYCODE_FOCUS = 80  # 对焦键
    KEYCODE_POWER = 26  # 电源键
    KEYCODE_NOTIFICATION = 83  # 通知键
    KEYCODE_MUTE = 91  # 话筒静音键
    KEYCODE_VOLUME_MUTE = 164  # 扬声器静音键
    KEYCODE_VOLUME_UP = 24  # 音量+键
    KEYCODE_VOLUME_DOWN = 25  # 音量-键
    KEYCODE_ENTER = 66  # 回车键
    KEYCODE_ESCAPE = 111  # ESC键
    KEYCODE_DPAD_CENTER = 23  # 导航键 >> 确定键
    KEYCODE_DPAD_UP = 19  # 导航键 >> 向上
    KEYCODE_DPAD_DOWN = 20  # 导航键 >> 向下
    KEYCODE_DPAD_LEFT = 21  # 导航键 >> 向左
    KEYCODE_DPAD_RIGHT = 22  # 导航键 >> 向右
    KEYCODE_MOVE_HOME = 122  # 光标移动到开始键
    KEYCODE_MOVE_END = 123  # 光标移动到末尾键
    KEYCODE_PAGE_UP = 92  # 向上翻页键
    KEYCODE_PAGE_DOWN = 93  # 向下翻页键
    KEYCODE_DEL = 67  # 退格键
    KEYCODE_FORWARD_DEL = 112  # 删除键
    KEYCODE_INSERT = 124  # 插入键
    KEYCODE_TAB = 61  # Tab键
    KEYCODE_NUM_LOCK = 143  # 小键盘锁
    KEYCODE_CAPS_LOCK = 115  # 大写锁定键
    KEYCODE_BREAK = 121  # Break / Pause键
    KEYCODE_SCROLL_LOCK = 116  # 滚动锁定键
    KEYCODE_ZOOM_IN = 168  # 放大键
    KEYCODE_ZOOM_OUT = 169  # 缩小键
    KEYCODE_0 = 7
    KEYCODE_1 = 8
    KEYCODE_2 = 9
    KEYCODE_3 = 10
    KEYCODE_4 = 11
    KEYCODE_5 = 12
    KEYCODE_6 = 13
    KEYCODE_7 = 14
    KEYCODE_8 = 15
    KEYCODE_9 = 16
    KEYCODE_A = 29
    KEYCODE_B = 30
    KEYCODE_C = 31
    KEYCODE_D = 32
    KEYCODE_E = 33
    KEYCODE_F = 34
    KEYCODE_G = 35
    KEYCODE_H = 36
    KEYCODE_I = 37
    KEYCODE_J = 38
    KEYCODE_K = 39
    KEYCODE_L = 40
    KEYCODE_M = 41
    KEYCODE_N = 42
    KEYCODE_O = 43
    KEYCODE_P = 44
    KEYCODE_Q = 45
    KEYCODE_R = 46
    KEYCODE_S = 47
    KEYCODE_T = 48
    KEYCODE_U = 49
    KEYCODE_V = 50
    KEYCODE_W = 51
    KEYCODE_X = 52
    KEYCODE_Y = 53
    KEYCODE_Z = 54
    KEYCODE_PLUS = 81  # +
    KEYCODE_MINUS = 69  # -
    KEYCODE_STAR = 17  # *
    KEYCODE_SLASH = 76  # /
    KEYCODE_EQUALS = 70  # =
    KEYCODE_AT = 77  # @
    KEYCODE_POUND = 18  # #
    KEYCODE_APOSTROPHE = 75  # '
    KEYCODE_BACKSLASH = 73  # \
    KEYCODE_COMMA = 55  # ,
    KEYCODE_PERIOD = 56  # .
    KEYCODE_LEFT_BRACKET = 71  # [
    KEYCODE_RIGHT_BRACKET = 72  # ]
    KEYCODE_SEMICOLON = 74  # ;
    KEYCODE_GRAVE = 68  # `
    KEYCODE_SPACE = 62  # 空格键
    KEYCODE_MEDIA_PLAY = 126  # 多媒体键 >> 播放
    KEYCODE_MEDIA_STOP = 86  # 多媒体键 >> 停止
    KEYCODE_MEDIA_PAUSE = 127  # 多媒体键 >> 暂停
    KEYCODE_MEDIA_PLAY_PAUSE = 85  # 多媒体键 >> 播放 / 暂停
    KEYCODE_MEDIA_FAST_FORWARD = 90  # 多媒体键 >> 快进
    KEYCODE_MEDIA_REWIND = 89  # 多媒体键 >> 快退
    KEYCODE_MEDIA_NEXT = 87  # 多媒体键 >> 下一首
    KEYCODE_MEDIA_PREVIOUS = 88  # 多媒体键 >> 上一首
    KEYCODE_MEDIA_CLOSE = 128  # 多媒体键 >> 关闭
    KEYCODE_MEDIA_EJECT = 129  # 多媒体键 >> 弹出
    KEYCODE_MEDIA_RECORD = 130  # 多媒体键 >> 录音
 
import zqpy
if __name__ == '__main__':
    adb = AdbTools()
    adb.screenshot()
    # print(zqpy.ADBService.GetAppCurr())


class ToolsServiceClass(object):
    def ToNumberCompany(self, number):
        if number > 100000000:
            return number/100000000 + "亿"
        elif number > 10000000:
            return number/10000000 + "千万"
        elif number > 1000000:
            return number/1000000 + "百万"
        elif number > 100000:
            return number/100000 + "十万"
        elif number > 10000:
            return number/10000 + "万"
        else:
            return number
    
    #https://www.runoob.com/markdown/md-block.html
    def SendNotify(self, title="zq_python", desc="", isMerge=True, key=None):
        from HttpService import HttpServiceClass
        if len(title + '-->' + desc) < 256 and isMerge:
            title = title + '-->'  + desc
        httpService = HttpServiceClass()
        params = {"text": title, "desp": desc}
        print("方糖发送内容：{}".format(params))
        try:
            urlRequest = httpService.GetUrlResetByRequests(key or "https://sc.ftqq.com/SCU63651T3c16cbd7d3aa23d56645aeddc3459c135d9aebde1ca0a.send", params=params, headers={'Connection':'close'}, verify = False)
            urlRequest.close()
        except BaseException as e:
            print("方糖内容发送出错： ", str(e))
            urlRequest.close()
        finally:
            pass


# -*- coding: utf-8 -*-
from threading import Timer
import time

class WaitExecuteItemClass(object):
    id = -1
    currTime = 0
    nextTime = -1
    waitTime = -1
    repeatNum = 0
    funcEvent = None
    completeEvent = None
    killEvent = None

    isKill = False
    isFinish = False
    currRepeatNum = 1

    def run(self):
        if self.isFinish:
            return False
        if self.nextTime == -1:
            self.nextTime = self.waitTime
        if self.waitTime == 0:
            if self.funcEvent:
                self.funcEvent()
        else:
            if self.currTime == self.nextTime or self.nextTime == 0:
                if self.funcEvent:
                    self.funcEvent()
                if self.repeatNum == -1 or self.currRepeatNum < self.repeatNum:
                    self.nextTime = self.currTime + self.waitTime
                    self.currRepeatNum += 1
                else:
                    if self.completeEvent:
                        self.completeEvent()
                    self.isFinish = True
                    return False

        self.currTime += 1
        if self.isKill:
            self.killEvent()
            return False
        return True


    def stop(self):
        self.isKill = True


class WaitExecutServiceClass(object):
    waitDic = None
    mainTimer = None
    currIndex = 0

    def __init__(self, isOpen = False):
        self.waitDic = {}
        if isOpen:
            self.Open()

    def Open(self):
        self.mainTimer = Timer(1, self.run, args=())
        self.mainTimer.start()

    def Close(self):
        try:
            for key in self.waitDic.keys():
                self.RemoveWiteExecute(key)
            self.mainTimer._stop()
        except BaseException as e:
            print(str(e))
    
    def AddWaitExecute(self, repeatNum, waitTime, funcEvent, completeEvent, killEvent):
        self.currIndex += 1
        waitExecuteItem = WaitExecuteItemClass()
        waitExecuteItem.id = self.currIndex
        waitExecuteItem.waitTime = waitTime
        waitExecuteItem.repeatNum = repeatNum
        waitExecuteItem.funcEvent = funcEvent
        waitExecuteItem.completeEvent = completeEvent
        waitExecuteItem.killEvent = killEvent
        self.waitDic[self.currIndex] = waitExecuteItem
        return waitExecuteItem

    def RemoveWiteExecute(self, id):
        if self.waitDic and self.waitDic.get(id):
            self.waitDic[id].stop()

    def RemoveWiteExecute_item(self, waitExecuteItemData):
        if waitExecuteItemData:
            self.RemoveWiteExecute(waitExecuteItemData.id)

    def GetWaitExecute(self, id):
        if self.waitDic and self.waitDic.get(id):
            return self.waitDic[id]
        return None

    def run(self):
        index = 0
        while True:
            time.sleep(1)
            index += 1
            if len(self.waitDic) == 0:
                continue
            notNextDicKeys = []
            for item in self.waitDic:
                if self.waitDic:
                    isNextRun = self.waitDic[item].run()
                    if not isNextRun:
                        notNextDicKeys.append(item)
            for item in notNextDicKeys:
                self.waitDic.pop(item)

if __name__ == "__main__":
    waitExecute = WaitExecutServiceClass()
    waitExecute.Open()
    def funcEvent():
        print('funcEvent')
    def completeEvent():
        print('completeEvent')
    def killEvent():
        print('killEvent')
    waitExecute.AddWaitExecute(-1,0,funcEvent,completeEvent,killEvent)
    waitExecute.AddWaitExecute(-1,0,funcEvent,completeEvent,killEvent)

    import time
    time.sleep(4)
    waitExecute.Close()



import threading
import inspect
import ctypes

class ThreadServiceClass(object):
    def __init__(self):
        self.threadDic = {}

    def AddThread(self, Target, IsImmRun, Name = ""):
        '''
        IsImmRun:是否立即运行
        '''
        if Target == None:
            return None
        id = len(self.threadDic)
        self.threadDic[id] = ThreadItemClass(Name,Target,IsImmRun,Id=id)
        return self.threadDic[id]
    
    def RomeThread(self, ThreadItemClass):
        if ThreadItemClass == None:
            return
        if ThreadItemClass.ThreadId in self.threadDic:
            self.threadDic.pop(ThreadItemClass.ThreadId)

    def GetThread(self, ThreadItemClass):
        if ThreadItemClass == None:
            return None
        self.threadDic.get(ThreadItemClass.ThreadId, None)
        
    def GetThreadById(self, ThreadId):
        self.threadDic.get(ThreadId, None)
        
    def GetThreadByName(self, Name):
        listThread = None
        for item in self.threadDic:
            if self.threadDic[item].ThreadName == Name:
                if not listThread:
                    listThread = []
                listThread.append(self.threadDic[item])
        return listThread

    def GetThreadCount(self):
        return len(self.threadDic)

    '''
    def ReRunAll(self):
        for itemThread in self.threadDic:
            threadItem = self.threadDic[itemThread]
            if threadItem :
                threadItem.Run()
    '''

class ThreadItemClass(object):
    def __init__(self, Name, Target, IsImmRun, Id = 0):
        self.ThreadName = Name
        self.ThreadId = Id
        self.Target = Target
        self.ThreadInstance = threading.Thread(name=Name, target=Target)
        self.lock = None
        if IsImmRun:
            self.Start()

    def Start(self):
        self.ThreadInstance.start()
        
    def Run(self, isKillRun = True):
        if self.ThreadInstance.isAlive():
            if isKillRun:
                self.Stop()
                self.ThreadInstance = threading.Thread(name=self.ThreadName, target=self.Target)
                self.Start()
        else:
            self.Start()

    def Stop(self):
        if not self.ThreadInstance:
            return
        self.ThreadInstance._stop()
        '''
        tid = self.ThreadInstance.ident
        exctype = SystemExit
        tid = ctypes.c_long(tid)
        if not inspect.isclass(exctype):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if res == 0:
            raise ValueError("invalid thread id")
        elif res != 1:
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")
        '''
    
    def Lock(self):
        self.lock = threading.Lock()
        
    def UnLock(self):
        if self.lock:
            self.lock.acquire()
            self.lock.release()

    def ChangeTarget(self, Target):
        self.ThreadInstance._target = Target

def Target():
    i = 0
    while i < 10:
        i += 1
        print(i)

if __name__ == "__main__":
    ThreadService = ThreadServiceClass()
    A = ThreadService.AddThread(Target, True, "dsda")
    B = ThreadService.AddThread(Target, True, "dsda1")
    # A.Lock()
    # A.UnLock()
    B.Run()
    B.Run()
    B.Run()


import time
import os
import subprocess
from LogService import LogServiceClass
from RegexService import RegexServiceClass
from enum import IntEnum

class ADBServiceClass(object):
    __Log = LogServiceClass(tag="ADBServiceClass")
    LogD = __Log.LogD
    LogW = __Log.LogW
    LogE = __Log.LogE
    __regex = RegexServiceClass()

    sleep = 0
    deviceName = None
    def __init__(self, sleep = 0):
        self.ChangeSleep(sleep)

    def ChnageDevice(self, device):
        self.deviceName = device or None

    # 改变停顿时间
    def ChangeSleep(self, sleep = 0):
        self.sleep = sleep

    # 获取复制的值
    def GetCopyValue(self, device = None):
        self.ExecuteADB("adb shell am startservice ca.zgrs.clipper/.ClipboardService", device=device)
        readLine = self.ExecuteADB("adb shell am broadcast -a clipper.get", device=device)
        if len(readLine) < 2:
            return ""
        resultData = str(readLine[1], 'utf8')
        resultData = self.__regex.GetRegexDoubleQuotation(resultData)
        return resultData

    # ADB安装应用
    def Install(self, appPath, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB("adb install {}".format(appPath), device=device)

    # 杀掉某APP
    def KillApp(self, packageName, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB('adb shell "am force-stop {}"'.format(packageName), device=device)        

    # 显示某个应用的 某层
    def ShowApp(self, appPackage, appLayer, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB("adb shell am start -n {}/{}".format(appPackage,appLayer), device=device)
    def ShowAppByList(self, dataList):
        if dataList and len(dataList) >= 2:
            return self.ShowApp(dataList[0], dataList[1])
        else:
            self.LogW("不存在 dataList，或者位数不够")

    # 查看当前所在的页面 属于的package 和 活动层，方便打开用
    def GetAppCurr(self, device = None):        
        time.sleep(self.sleep)
        readLine = self.ExecuteADB('adb shell dumpsys window windows | findstr "Current"', device=device,shell=True,stdout=subprocess.PIPE)
        print(readLine)
        return readLine

    # ADB点击
    def Click(self, x, y, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB("adb shell input tap {} {}".format(x,y), device=device)
    def ClickByList(self, dataList):
        if dataList and len(dataList) >= 2:
            return self.Click(dataList[0],dataList[1])
        else:
            self.LogW("不存在 dataList，或者位数不够")

    # ADB按住滑动
    def DragAndDrop(self, x1, y1, x2, y2, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB("adb shell input draganddrop {} {} {} {}".format(x1, y1, x2, y2), device=device)
    def DragAndDropByList(self, dataList):
        if dataList and len(dataList) >= 4:
            return self.DragAndDrop(dataList[0], dataList[1], dataList[2], dataList[3])
        else:
            self.LogW("不存在 dataList，或者位数不够")

    # ADB滑动
    def Swipe(self, x1, y1, x2, y2, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB("adb shell input swipe {} {} {} {}".format(x1, y1, x2, y2), device=device)
    def SwipeByList(self, dataList):
        if dataList and len(dataList) >= 4:
            return self.Swipe(dataList[0], dataList[1], dataList[2], dataList[3])
        else:
            self.LogW("不存在 dataList，或者位数不够")

    # 输入键
    def InputKeyEvent(self, content = "", device = None):
        time.sleep(self.sleep)        
        return self.ExecuteADB("adb shell input keyevent '{}'".format(content), device=device)

    # 输入内容，不支持中文  模拟器以 /mnt开头 + /sdcard手机最高目录名字
    def InputText(self, content = "", device = None):
        time.sleep(self.sleep)        
        return self.ExecuteADB("adb shell input text '{}'".format(content), device=device)
        # return self.ExecuteADB("adb shell am broadcast -a ADB_INPUT_TEXT --es msg '{}'".format(content))

    # 输入内容，支持中文  必须满足 1.安装了ADBKeyboard.apk  2.adb 1.0.32以上
    def InputTextByApp(self, content = "", device = None):
        time.sleep(self.sleep)        
        return self.ExecuteADB("adb shell am broadcast -a ADB_INPUT_TEXT --es msg '{}'".format(content), device=device)

    # 电脑文件复制到手机上，手机测不支持中文
    def PushFile(self, pcPath, androidPath, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB("adb push {} {}".format(pcPath, androidPath), device=device,stdout = subprocess.DEVNULL) # 小文件可以用subprocess.PIPE，大文件用它要崩，一直卡

    # 手机文件复制到电脑上，手机测不支持中文
    def PullFile(self, androidPath, pcPath, device = None):
        time.sleep(self.sleep)
        return self.ExecuteADB("adb pull {} {}".format(androidPath, pcPath), device=device)
    # 手机截屏
    def Screenshot(self, targetPath='', device = None):
        """
        手机截图
        :param targetPath: 目标路径
        :return:
        """
        format_time = int(time.time())
        self.ExecuteADB('adb shell screencap -p /sdcard/%s.png'%format_time, device=device)
        time.sleep(1)
        outPath = targetPath
        if targetPath == '':
            outPath = os.path.dirname(__file__)
        self.PullFile('/sdcard/%s.png' % (format_time,), outPath)
        self.RemoveFile('/sdcard/%s.png' % (format_time,))
        time.sleep(1)
        return outPath if ('.jpg' in outPath or '.png' in outPath) else ('%s%s.png' % (outPath, format_time,))
    # 移除手机文件
    def RemoveFile(self, path):
        """
        从手机端删除文件
        :return:
        """
        self.ExecuteADB('adb shell rm %s'%path)
    # 获取设备信息列表
    def Devices(self, device = None):
        time.sleep(self.sleep)
        devices = []
        readLine = self.ExecuteADB("adb devices", device=device) 
        for item in readLine:
            item = item.decode('utf-8')
            items = item.split('\t')
            if len(items) == 2:
                newItems = []
                for itemTemp in items:
                    itemTemp = itemTemp.strip().replace("-","_b_")
                    itemTemp = itemTemp.replace(".","_c_")
                    itemTemp = itemTemp.replace(":","_d_")
                    if itemTemp.split('_')[0].isdigit():
                        itemTemp = "_a_" + itemTemp
                    newItems.append(itemTemp)
                devices.append(newItems)
        return devices
    # 打开Uri
    def OpenUri(self, uri, device=None):
        time.sleep(self.sleep)
        return self.ExecuteADB('adb shell am start -a android.intent.action.VIEW -d '%uri, device=device)

    def ExecuteADB(self, adbCommand, device = None, shell = True, stdout = subprocess.PIPE):
        deviceTemp = device or self.deviceName
        if deviceTemp:
            deviceTemp = deviceTemp.replace("_a_","")
            deviceTemp = deviceTemp.replace("_b_","-")
            deviceTemp = deviceTemp.replace("_c_",".")
            deviceTemp = deviceTemp.replace("_d_",":")
            adbCommand = adbCommand.replace("adb ", "adb -s {} ".format(deviceTemp))
        popen = subprocess.Popen(adbCommand,shell=shell,stdout=stdout)
        popen.wait()
        if stdout == subprocess.PIPE:
            readLine = popen.stdout.readlines()
            return readLine
        else:
            return True

class ABDKeyEnum(IntEnum):
    KEYCODE_UNKNOWN = 0,
    KEYCODE_MENU = 1, 
    KEYCODE_SOFT_RIGHT = 2, 
    KEYCODE_HOME = 3, 
    KEYCODE_BACK = 4, 
    KEYCODE_CALL = 5, 
    KEYCODE_ENDCALL = 6, 
    KEYCODE_0 = 7, 
    KEYCODE_1 = 8, 
    KEYCODE_2 = 9, 
    KEYCODE_3 = 10,
    KEYCODE_4 = 11,
    KEYCODE_5 = 12,
    KEYCODE_6 = 13,
    KEYCODE_7 = 14,
    KEYCODE_8 = 15,
    KEYCODE_9 = 16,
    KEYCODE_STAR = 17,
    KEYCODE_POUND = 18,
    KEYCODE_DPAD_UP = 19,
    KEYCODE_DPAD_DOWN = 20,
    KEYCODE_DPAD_LEFT = 21,
    KEYCODE_DPAD_RIGHT = 22,
    KEYCODE_DPAD_CENTER = 23,
    KEYCODE_VOLUME_UP = 24,
    KEYCODE_VOLUME_DOWN = 25,
    KEYCODE_POWER = 26,
    KEYCODE_CAMERA = 27,
    KEYCODE_CLEAR = 28,
    KEYCODE_A = 29,
    KEYCODE_B = 30,
    KEYCODE_C = 31,
    KEYCODE_D = 32,
    KEYCODE_E = 33,
    KEYCODE_F = 34,
    KEYCODE_G = 35,
    KEYCODE_H = 36,
    KEYCODE_I = 37,
    KEYCODE_J = 38,
    KEYCODE_K = 39,
    KEYCODE_L = 40,
    KEYCODE_M = 41,
    KEYCODE_N = 42,
    KEYCODE_O = 43,
    KEYCODE_P = 44,
    KEYCODE_Q = 45,
    KEYCODE_R = 46,
    KEYCODE_S = 47,
    KEYCODE_T = 48,
    KEYCODE_U = 49,
    KEYCODE_V = 50,
    KEYCODE_W = 51,
    KEYCODE_X = 52,
    KEYCODE_Y = 53,
    KEYCODE_Z = 54,
    KEYCODE_COMMA = 55,
    KEYCODE_PERIOD = 56,
    KEYCODE_ALT_LEFT = 57,
    KEYCODE_ALT_RIGHT = 58,
    KEYCODE_SHIFT_LEFT = 59,
    KEYCODE_SHIFT_RIGHT = 60,
    KEYCODE_TAB = 61,
    KEYCODE_SPACE = 62,
    KEYCODE_SYM = 63,
    KEYCODE_EXPLORER = 64,
    KEYCODE_ENVELOPE = 65,
    KEYCODE_ENTER = 66,
    KEYCODE_DEL = 67,
    KEYCODE_GRAVE = 68,
    KEYCODE_MINUS = 69,
    KEYCODE_EQUALS = 70,
    KEYCODE_LEFT_BRACKET = 71,
    KEYCODE_RIGHT_BRACKET = 72,
    KEYCODE_BACKSLASH = 73,
    KEYCODE_SEMICOLON = 74,
    KEYCODE_APOSTROPHE = 75,
    KEYCODE_SLASH = 76,
    KEYCODE_AT = 77,
    KEYCODE_NUM = 78,
    KEYCODE_HEADSETHOOK = 79,
    KEYCODE_FOCUS = 80,
    KEYCODE_PLUS = 81,
    KEYCODE_MENU_2 = 82,
    KEYCODE_NOTIFICATION = 83,
    KEYCODE_SEARCH = 84,
    TAG_LAST_KEYCODE = 85,
    # 话筒静音键
    KEYCODE_MUTE = 91,
    # 向上翻页键
    KEYCODE_PAGE_UP = 92,
    # 向下翻页键
    KEYCODE_PAGE_DOWN = 93,
    # ESC键
    KEYCODE_ESCAPE = 111,
    # 删除键
    KEYCODE_FORWARD_DEL = 112,
    # 大写锁定键
    KEYCODE_CAPS_LOCK = 115,
    # 滚动锁定键
    KEYCODE_SCROLL_LOCK = 116,
    # Break/Pause键
    KEYCODE_BREAK = 121,
    # 光标移动到开始键
    KEYCODE_MOVE_HOME = 122,
    # 光标移动到末尾键
    KEYCODE_MOVE_END = 123,
    # 插入键
    KEYCODE_INSERT = 124,
    # 小键盘锁
    KEYCODE_NUM_LOCK = 143,
    # 扬声器静音键
    KEYCODE_VOLUME_MUTE = 164,
    # 放大键
    KEYCODE_ZOOM_IN = 168,
    # 缩小键
    KEYCODE_ZOOM_OUT = 169,


import urllib
import requests
import json
import os
from LogService import LogServiceClass
import random,os,time

class HttpServiceClass(object):
    Log = LogServiceClass(tag="HttpServiceClass")
    LogD = Log.LogD
    LogW = Log.LogW
    LogE = Log.LogE
    # 'accept-encoding': 'gzip, deflate, br',
    headers = {
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
        'accept-language': 'zh-CN,zh;q=0.9',
        'pragma': 'no-cache',
        'cache-control': 'no-cache',
        'upgrade-insecure-requests': '1',
        # 'user-agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1'
        'user-agent': 'Mozilla/5.0 (Linux; U; Android 5.1.1; zh-cn; MI 4S Build/LMY47V) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/53.0.2785.146 Mobile Safari/537.36 XiaoMi/MiuiBrowser/9.1.3',
    }

    params = {}

    def __init__(self):
        pass
    
    def GetUrlReset(self, url):
        res = urllib.request.urlopen(url, data=None, timeout=60)
        return res.read().decode()
    
    def GetUrlResetDataByRequests(self, url, headers = None, params = None, data = None, stream = False, verify=False):
        res = self.GetUrlResetByRequests(url, headers, params, data, stream, verify)
        dumpTab = None
        try:
            if res.ok:
                try:
                    if res.text != '':
                        dumpTab = json.loads(res.text)
                except BaseException as e:
                    self.LogE("GetUrlResetDataByRequests error ==>>  " + str(e))
                finally:
                    pass
            else:
                self.LogE("GetUrlResetDataByRequests error ==>>  " + res.text)
        except BaseException as e:            
            self.LogE("GetUrlResetDataByRequests BaseException error ==>> URL: {}  Error: {}".format(url, str(e)))
        return dumpTab
    
    def GetUrlResetByRequests(self, url, headers = None, params = None, data = None, stream = False, verify=False):
        requests.adapters.DEFAULT_RETRIES = 5
        s = requests.session()
        s.keep_alive = False
        res = None
        
        try:
            if params:
                for paramItem in params:
                    self.params[paramItem] = params[paramItem]
            if headers:
                for headerItem in headers:
                    self.headers[headerItem] = headers[headerItem]
            timeInt = int(time.time())
            timeLong = timeInt*1000

            self.params.setdefault('ts', timeInt)
            self.params.setdefault('_rticket', timeLong)
            self.params.setdefault('channel', 'Meizu{}'.format(range(0,9999)))
            self.params.setdefault('device_type', 'ANDROID{}'.format(range(0,9999)))
            self.params.setdefault('device_platform', 'android'.format(range(0,9999)))

            requests.packages.urllib3.disable_warnings()
            res = s.get(url, headers = self.headers, params = self.params, data = data, verify=verify, stream=stream)
            res.close()            
            return res
        except BaseException as e:            
            self.LogE("GetUrlResetByRequests BaseException error ==>> URL: {}  Error: {}".format(url, str(e)))        
        return res
    

        
    def PostUrlResetDataByRequests(self, url, headers = None, params = None, data = None, stream = False, files= False, verify=False, cookies=False):
        res = self.PostUrlResetByRequests(url, headers, params, data, stream, files, verify, cookies)
        dumpTab = None
        try:
            if res.ok:
                try:
                    if res.text != '':
                        dumpTab = json.loads(res.text)
                except BaseException as e:
                    self.LogE("PostUrlResetDataByRequests error ==>>  " + str(e))
                finally:
                    pass
            else:
                self.LogE("PostUrlResetDataByRequests error ==>>  " + res.text)
        except BaseException as e:            
            self.LogE("PostUrlResetDataByRequests BaseException error ==>> URL: {}  Error: {}".format(url, str(e)))
        return dumpTab

    def PostUrlResetByRequests(self, url, headers = None, params = None, data = None, stream = False, files= False, verify=False, cookies=False):
        requests.adapters.DEFAULT_RETRIES = 5
        s = requests.session()
        s.keep_alive = False
        res = None
        
        try:
            if params:
                for paramItem in params:
                    self.params[paramItem] = params[paramItem]
            if headers:
                for headerItem in headers:
                    self.headers[headerItem] = headers[headerItem]
            timeInt = int(time.time())
            timeLong = timeInt*1000

            self.params.setdefault('ts', timeInt)
            self.params.setdefault('_rticket', timeLong)
            self.params.setdefault('channel', 'Meizu{}'.format(range(0,9999)))
            self.params.setdefault('device_type', 'ANDROID{}'.format(range(0,9999)))
            self.params.setdefault('device_platform', 'android'.format(range(0,9999)))

            requests.packages.urllib3.disable_warnings()
            res = s.post(url, headers = self.headers, params = self.params, data = data, verify=verify, stream=stream, files= files, cookies=cookies)
            res.close()            
            return res
        except BaseException as e:            
            self.LogE("PostUrlResetByRequests BaseException error ==>> URL: {}  Error: {}".format(url, str(e)))        
        return res
    


from contextlib import closing
import requests
import os
from LogService import LogServiceClass

class VideoDownloadServiceClass(object):
    Log = LogServiceClass(tag="VideoDownloadServiceClass")
    LogD = Log.LogD
    LogW = Log.LogW
    LogE = Log.LogE
    headers = {
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
        'accept-language': 'zh-CN,zh;q=0.9',
        'pragma': 'no-cache',
        'cache-control': 'no-cache',
        'upgrade-insecure-requests': '1',
        'user-agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1'
        # 'user-agent': 'Mozilla/5.0 (Linux; U; Android 5.1.1; zh-cn; MI 4S Build/LMY47V) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/53.0.2785.146 Mobile Safari/537.36 XiaoMi/MiuiBrowser/9.1.3',
    }
    def __init__(self, *args):
        pass
    
    def Down(self, urlPath, saveFilePath):
        try:
            requests.adapters.DEFAULT_RETRIES = 5
            s = requests.session()
            s.keep_alive = False
            with closing(requests.get(urlPath, headers = self.headers, stream=True)) as response:
                if not response.ok:
                    print('错误页面：==>>>  ' + response.text)
                    return False, saveFilePath
                chunk_size = 1024
                content_size = int(response.headers['content-length'])
                if(os.path.exists(saveFilePath) and os.path.getsize(saveFilePath)==content_size):
                    print('跳过'+saveFilePath)
                    return False, saveFilePath
                else:
                    if not ".mp4" in saveFilePath.lower():
                        saveFilePath = saveFilePath + ".mp4" 
                    videoDirPath = os.path.dirname(saveFilePath)
                    if not os.path.exists(videoDirPath):
                        os.makedirs(videoDirPath)
                    progress = ProgressBar(saveFilePath, total=content_size, unit="KB", chunk_size=chunk_size, run_status="正在下载",fin_status="下载完成")
                    with open(saveFilePath, "wb") as file:
                        for data in response.iter_content(chunk_size=chunk_size):
                            file.write(data)
                            progress.refresh(count=len(data))
            return True, saveFilePath
        except BaseException as e:
            self.LogE("Down error ==>>  " + str(e))
            return False, saveFilePath

#下载进度
class ProgressBar(object):
    def __init__(self, title, count=0.0, run_status=None, fin_status=None, total=100.0, unit='', sep='/',
                 chunk_size=1.0):
        super(ProgressBar, self).__init__()
        self.info = "[%s] %s %.2f %s %s %.2f %s"
        self.title = title
        self.total = total
        self.count = count
        self.chunk_size = chunk_size
        self.status = run_status or ""
        self.fin_status = fin_status or " " * len(self.statue)
        self.unit = unit
        self.seq = sep

    def __get_info(self):
        # 【名称】状态 进度 单位 分割线 总数 单位
        _info = self.info % (
            self.title, self.status, self.count / self.chunk_size, self.unit, self.seq, self.total / self.chunk_size,
            self.unit)
        return _info

    def refresh(self, count=1, status=None):
        self.count += count
        # if status is not None:
        self.status = status or self.status
        end_str = "\r"
        if self.count >= self.total:
            end_str = '\n'
            self.status = status or self.fin_status
        print(self.__get_info(), end=end_str)

if __name__ == "__main__":
    VideoDownloadService = VideoDownloadServiceClass()
    videoUrl = "http://v3-dy-y.bytecdn.cn/8049399863f5611a52a58c972ecb8f72/5d0f208b/video/m/220c5c31d4d72d9426fb1f822015dfdc13811625bf7f000067d0dc44f3d8/?rc=MzRpOjY2NTh3bTMzZ2kzQ8ODkzOTUzNDk6NjY5PDNAKXUpQGczdSlAZjN2KUBmaHV5cTFmc2hoZGY7NEBmaWNeL3MyaHBfLS0wLS9zczVvI28jNi8uNS4tLi0tLzEuLy0uL2k6Yi5wIzphLXEjOmAwbyNwYmZyaF4ranQ6Iy8uXg%3D%3D"
    VideoDownloadService.Down(videoUrl, "Video/测试.mp4")


import sqlite3

class SqliteServiceClass(object):
    def __init__(self):
        self.conn = None
        self.cursor = None

    def OpenSql(self, dbName):
        '''打开/创建 数据库'''
        try:
            # global conn, cursor
            self.conn = sqlite3.connect(dbName)
            self.cursor = self.conn.cursor()
            return True
        except BaseException as e:
            print(str(e))
            return False

    def CloseSql(self):
        '''关闭数据库'''
        try:
            if self.cursor:
                self.cursor.close()
            if self.conn:
                self.conn.close()
            return True
        except BaseException as e:
            print(str(e))
            return False

    def CreateTable(self, tabName, fields, attribute, size, defaultValue, isUniqueId):
        '''创建表'''
        try:
            if not self.cursor:
                return False
            strValue = ''
            for index in range(0, len(fields)):
                strValue = strValue + '%s %s %s %s %s '%(fields[index], attribute[index], '' if not size or len(size) < len(fields) else '(%s)'%size[index], '' if index!=0 else 'primary key %s'%('UNIQUE' if isUniqueId else ''), ',' if index!=len(fields)-1 else '') #defaultValue[index]
            self.cursor.execute('create table %s (%s)'%(tabName, strValue))
            return True
        except BaseException as e:
            print(str(e))
            return False

    def DestoryTable(self, tabName):
        '''删除表'''
        try:
            if not self.cursor:
                return False
            self.cursor.execute('drop table %s'%(tabName))
            self.conn.commit()
            return True
        except BaseException as e:
            print(str(e))
            return False

    def InsertData(self, tabName, values):
        '''
        插入数据
        values -> [(),()]
        '''
        try:
            if not self.cursor:
                return False
            strValue = ''
            for index in range(0, len(values[0])):
                strValue = strValue + ('?' if index==0 else ',?')
            self.cursor.executemany('INSERT INTO %s VALUES (%s)'%(tabName, strValue), values)
            self.conn.commit()
            return True
        except BaseException as e:
            print(str(e))
            return False

    def DelData(self, tabName, key, value):
        '''删除数据'''
        try:
            if not self.cursor:
                return False
            self.cursor.execute('DELETE FROM %s WHERE %s=%s'%(tabName, key, value))
            self.conn.commit()
            return True
        except BaseException as e:
            print(str(e))
            return False

    def UpdateData(self, tabName, key, value, fields, values):
        '''更新数据'''
        try:
            if not self.cursor:
                return False
            strValue = ''
            for index in range(0, len(fields)):
                strValue = strValue + ('%s=%s,' if index!=len(fields)-1 else '%s=%s')%(fields[index], values[index])   
            self.cursor.execute('UPDATE %s SET %s WHERE %s=%s'%(tabName, strValue, key, value))
            self.conn.commit()
            return True
        except BaseException as e:
            print(str(e))
            return False

    def SelectData(self, tabName, fields, keys, values, marks = False):       
        '''
        查询数据
        marks [1,-1] 1 and  -1 or
        '''
        try:
            if not self.cursor:
                return False
            strValue = ''
            for index in range(0, len(fields)):
                strValue = strValue + ('%s,' if index!=len(fields)-1 else '%s')%(fields[index])
            whereValue = ''
            if keys and values:
                for index in range(0, len(keys)):
                    whereValue = whereValue + ('WHERE ' if index==0 else '') + ('%s=%s '%(str(keys[index]),str(values[index]))) + ('' if not marks or index>=len(marks) else ('and ' if marks[index]==1 else 'or ' ))
            self.cursor.execute(('SELECT %s FROM %s '%('*' if len(fields)==0 else strValue, tabName)) + whereValue)
            return self.cursor.fetchall()
        except BaseException as e:
            print(str(e))
            return []
            
    def SelectSingData(self, tabName, fields, keys, values, marks = False):       
        '''
        查询满足的单行数据
        marks [1,-1] 1 and  -1 or
        '''
        data = self.SelectData(tabName, fields, keys, values, marks)
        return None if len(data) == 0 else data[0]

    def GetTableRecordNum(self, tabName):
        '''获取表中记录数'''
        try:
            if not self.cursor:
                return False
            self.cursor.execute('SELECT count(*) FROM %s'%(tabName))
            return self.cursor.fetchone()[0]
        except BaseException as e:
            print(str(e))
            return 0

    def IsExistsTable(self, tabName):
        '''是否存在表'''
        try:
            if not self.cursor:
                return False
            self.cursor.execute('SELECT count(*) FROM %s'%(tabName))
            return len(self.cursor.fetchall()) != 0
        except BaseException as e:
            print(str(e))
            return False

    def IsExistsData(self, tabName, keys, values):
        '''数据是否存在于表中'''
        try:
            if not self.cursor:
                return False
            selectData = self.SelectData(tabName, [], keys, values)
            return selectData and len(selectData) != 0
        except BaseException as e:
            print(str(e))
            return False

if __name__ == "__main__":
    sql = SqliteServiceClass()
    print('%s -> %s'%('OpenSql', sql.OpenSql('Test.db')))
    print('%s -> %s'%('CreateTable', sql.CreateTable('TestTable', ['id', 'name'], ['int', 'text'], [10, 20], [], True)))
    print('%s -> %s'%('InsertData', sql.InsertData('TestTable', [(1,'5555'),(2,'666'),(3,'777'),(5,'5555999')])))
    print('%s -> %s'%('GetTableRecordNum', sql.GetTableRecordNum('TestTable')))
    print('%s -> %s'%('IsExistsTable', sql.IsExistsTable('TestTable')))
    print('%s -> %s'%('IsExistsTable', sql.IsExistsTable('TestTable1')))
    print('%s -> %s'%('IsExistsData', sql.IsExistsData('TestTable', ['id'], [1])))
    print('%s -> %s'%('UpdateData',sql.UpdateData('TestTable', 'id', 1, ['id', 'name'],[11,'11'])))
    print('%s -> %s'%('DelData', sql.DelData('TestTable', 'id', 11)))
    print('%s -> %s'%('SelectData', sql.SelectData('TestTable', ['id','name'], ['id'], [5])))
    print('%s -> %s'%('SelectData', sql.SelectData('TestTable', [], False, False)))
    print('%s -> %s'%('DestoryTable', sql.DestoryTable('TestTable')))
    print('%s -> %s'%('CloseSql', sql.CloseSql()))
    import os
    os.remove('Test.db')


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'zqpy'
DESCRIPTION = 'const tools manager'
URL = 'https://gitee.com/qingBB/zqpy'
EMAIL = '1620829248@qq.com'
AUTHOR = 'ZhouQing'
REQUIRES_PYTHON = '>=3.5.0'
# VERSION = '0.1.2'     # use in __version__.py
VERSION = False

# run zqpy module, require package
REQUIRED = [
    'requests', 'setuptools', 'qrcode', 'Pillow', 'natsort',
    'opencv-python', 'aircv', 'pyautogui', 'appium-python-client', 'pytesseract', 'jieba',
]

# opertion require package
EXTRAS = {
    # 'fancy feature': ['django'],
}

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION


class UploadCommand(Command):
    """setup.py  upload"""

    description = 'create and publish package.'
    user_options = []

    @staticmethod
    def status(s):
        """print"""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('del before create')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('build source wheel(general) distribution')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('pass twine upload PyPI')
        os.system('twine upload dist/*')

        self.status('push to git')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')
        os.system('git add .')
        os.system('git stage')
        os.system('git commit -m "{0}{1}"'.format(about['__version__'], input('请输入提交日志信息-> ')))
        os.system('git push')

        # 自动升级下一个小版本
        import zqpy
        versionPath = os.path.join(here, project_slug, '__version__.py')
        strValue = zqpy.FileService.ReadFile(versionPath)
        versionList = about['__version__'].split('.')
        strValue = strValue.replace(versionList[2],str(int(versionList[2])+1))
        zqpy.FileService.WriteFile(versionPath, strValue)
        print('自动版本号+1  完成\n\n')

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    zip_safe=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
    },
)


import smtplib
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.header import Header

from ToolsService import ToolsServiceClass
tools = ToolsServiceClass()

class MailServiceClass(object):
    def __init__(self):
        self.mail_host="smtp.qq.com"  #设置服务器
        self.mail_user="1620829248@qq.com"    #用户名
        self.mail_pass="askqrqhzcusfcddf"   #口令 
        self.sender = '1620829248@qq.com'
        self.receiversInit = ['877665041@qq.com']  # 接收邮件，可设置为你的QQ邮箱或者其他邮箱
    
    def SendMail(self, userName, title, content, receivers):
        msgRoot = MIMEMultipart('related')
        msgRoot['From'] = Header(userName, 'utf-8')
        msgRoot['To'] =  Header("用户", 'utf-8')
        subject = title #'Python SMTP 邮件测试'
        msgRoot['Subject'] = Header(subject, 'utf-8')
        
        msgAlternative = MIMEMultipart('alternative')
        msgRoot.attach(msgAlternative)
        
        
        mail_msg = """
        <p>Python 邮件发送测试...</p>
        <p><a href="http://vip.mayishidai.cn">菜鸟教程链接</a></p>
        <p>图片演示：</p>
        <p><img src="cid:image1"></p>
        """
        # msgAlternative.attach(MIMEText(mail_msg, 'html', 'utf-8'))
        msgAlternative.attach(MIMEText(content, 'html', 'utf-8'))
        
        '''
        fp = open('./image/test.jpg', 'rb')
        msgImage = MIMEImage(fp.read())
        fp.close()
        
        # 定义图片 ID，在 HTML 文本中引用
        msgImage.add_header('Content-ID', '<image1>')
        msgRoot.attach(msgImage)
        '''

        receiversTemp = []
        receiversTemp.extend(self.receiversInit)
        receiversTemp.extend(receivers)
        reTitle = title
        if len(title + content) < 256 :
        	reTitle = title + content
        try:
            smtpObj = smtplib.SMTP() 
            smtpObj.connect(self.mail_host, 25)    # 25 为 SMTP 端口号
            smtpObj.login(self.mail_user,self.mail_pass)
            smtpObj.sendmail(self.sender, receivers, msgRoot.as_string())
            print ("邮件发送成功")
            tools.SendNotify("邮件：{}".format(reTitle), "{}".format(content))
            return {True, "邮件发送成功"}
        except smtplib.SMTPException as e:
            print ("Error: 无法发送邮件" + str(e))
            tools.SendNotify("邮件：{}".format(reTitle), "{}  \n  {}".format(content,str(e)))
            return {False, "Error: 无法发送邮件"}

if __name__ == "__main__":
    MailService = MailServiceClass()
    content = """
        <p>Python 邮件发送测试...</p>
        <p><a href="http://vip.mayishidai.cn">菜鸟教程链接</a></p>
        <p>图片演示：</p>
        <p><img src="cid:image1"></p>
        """
    MailService.SendMail("用户","嘿嘿嘿",content,"877665041@qq.com")
    MailServiceClass().SendMail("用户","嘿嘿嘿",content,"877665041@qq.com")



import re

class RegexServiceClass(object):

    # 获取URL  返回数组
    def GetRegexUrl(self, content):
        urlRegex = re.findall('https?://(?:[-\w.]|(?:%[\da-fA-F]{2})*/)+', content)
        return urlRegex 
    
    # 获取"" 中的内容  返回数组
    def GetRegexDoubleQuotation(self, content):
        pattern = re.compile('"(.*)"')
        resultData = pattern.findall(content)
        return resultData 
    
    # 获取a b 之间的内容  返回数组
    def GetRegexBetweenContent(self, a, b, content):
        result = re.findall(".*%s(.*)%s.*"%(a,b),content)
        '''
        print(result)
        for x in result:
            print(x)
        '''
        return result